@@ -443,6 +443,12 @@ class PolicyViolationError(BrokerResponseError):
     description = 'Request parameters do not satisfy the configured policy.'
 
 
+class SecurityDisabledError(BrokerResponseError):
+    errno = 54
+    message = 'SECURITY_DISABLED'
+    description = 'Security features are disabled.'
+
+
 class KafkaUnavailableError(KafkaError):
     pass
 
@@ -462,26 +468,6 @@ class KafkaConnectionError(KafkaError):
     invalid_metadata = True
 
 
-class ConnectionError(KafkaConnectionError):
-    """Deprecated"""
-
-
-class BufferUnderflowError(KafkaError):
-    pass
-
-
-class ChecksumError(KafkaError):
-    pass
-
-
-class ConsumerFetchSizeTooSmall(KafkaError):
-    pass
-
-
-class ConsumerNoMoreData(KafkaError):
-    pass
-
-
 class ProtocolError(KafkaError):
     pass
 
